//
// Copyright 2004 Alan Post
//
// This file is part of voudp.
//
// voudp is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// voudp is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
// A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// voudp; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
//

#ifndef __RECEIVER_H__
#define __RECEIVER_H__

#include "constants.h"
#include <map>
#include <set>
#include <sys/time.h>

class frame_queue_t;
struct cmp_fqs_by_time_t {
    bool operator()( frame_queue_t *a, frame_queue_t *b );
};
typedef std::map<client_id_t, frame_queue_t*> frame_queues_t;
typedef std::set<frame_queue_t*, cmp_fqs_by_time_t> fqs_by_time_t;

class Receiver
{
public:
    Receiver( const int    &port,
              const size_t &max_ms_lag,
              const size_t &max_ms_guess );
    ~Receiver();
    int get_sock_fd() const;
    void recv_packet();
    void decode_frame( float *frame_acc );
    void print_diagnostics() const;
private:
    frame_queues_t *p_fqs;
    fqs_by_time_t *p_fqs_by_time;
    size_t max_f_lag_default;
    timeval max_guess;
    int recv_fd;

    size_t n_f_silence;
    size_t n_f_sound;
};

#endif // __RECEIVER_H__
